<?php
require __DIR__ . '/../sig.php';

if(is_file(NP_INFO)) {
    $nowPlaying = true;
    $nowPlayingHasCover = is_file(NP_COVER_IMG);
    $npInfo = unserialize(file_get_contents(NP_INFO));
} else $nowPlaying = $nowPlayingHasCover = false;

$reqMethod = $_SERVER['REQUEST_METHOD'];
$reqPath = '/' . trim(parse_url($_SERVER['REQUEST_URI'], PHP_URL_PATH), '/');

if($reqMethod !== 'HEAD' && $reqMethod !== 'GET') {
    http_response_code(405);
    header('Allow: GET, HEAD');
    return;
}

if($reqPath === '/source' || $reqPath === '/sig-src.php') {
    http_response_code(301);
    header('Location: https://patchii.net/flash/signature');
    return;
}

if($reqPath === '/signature.png') {
    $ip_000 = new ImagickPixel('#000');
    $ip_444 = new ImagickPixel('#444');
    $ip_fff = new ImagickPixel('#fff');

    $imagick = new Imagick;
    $imagick->newImage(SIG_WIDTH, SIG_HEIGHT, $ip_000, 'png');

    $background = new Imagick(BG_PATH . DIRECTORY_SEPARATOR . BG_IMAGE);
    $imagick->compositeImage($background, Imagick::COMPOSITE_COPY, 0, 0);
    $background->destroy();

    $footer = new Imagick(SIG_FOOTER);
    $footerHeight = $footer->getImageHeight() - ($nowPlaying ? 0 : 10);

    $imagick->compositeImage($footer, Imagick::COMPOSITE_MATHEMATICS, 0, SIG_HEIGHT - $footerHeight);
    $footer->destroy();

    $imagickDraw = new ImagickDraw;
    $imagickDraw->setFillColor($ip_444);
    $imagickDraw->setFont(FW_FONT_FAM);
    $imagickDraw->setFontSize(FW_FONT_SIZE);

    if($nowPlayingHasCover) {
        try {
            $imagickDraw->rectangle(
                SIG_WIDTH - (NP_COVER_RES + NP_COVER_PAD) - NP_COVER_FRAME,
                NP_COVER_PAD - NP_COVER_FRAME,
                SIG_WIDTH - (NP_COVER_RES + NP_COVER_PAD) + NP_COVER_RES,
                NP_COVER_PAD + NP_COVER_RES,
            );

            $npCover = new Imagick(NP_COVER_IMG);
            $npCover->resizeImage(NP_COVER_RES, NP_COVER_RES, Imagick::FILTER_CATROM, 0.9, true);

            $imagick->drawImage($imagickDraw);
            $imagick->compositeImage($npCover, Imagick::COMPOSITE_COPY, SIG_WIDTH - (NP_COVER_RES + NP_COVER_PAD), NP_COVER_PAD);

            $npCover->destroy();
        } catch(ImagickException $ex) {
            $nowPlayingHasCover = false;
        }
    }

    $imagickDraw->setFillColor(BG_IMAGE === 'bg4.png' ? $ip_000 : $ip_fff);

    $imagickLogo = new Imagick();
    $imagickLogo->newImage($imagick->getImageWidth(), $imagick->getImageHeight(), new ImagickPixel('none'), 'png');
    $imagickLogo->annotateImage(
        $imagickDraw,
        SIG_WIDTH - (4.8 * FW_FONT_SIZE) - ($nowPlayingHasCover ? (NP_COVER_RES + NP_COVER_PAD + NP_COVER_FRAME) : 0),
        SIG_HEIGHT - ($nowPlaying ? 37 : 27),
        0,
        FW_TEXT
    );

    $imagickShadow = clone $imagickLogo;
    $imagickShadow->shadowImage(.8, 5, 0, 0);
    $imagickShadow->compositeImage($imagickLogo, Imagick::COMPOSITE_OVER, 4, 2);
    $imagickLogo->destroy();

    $imagick->compositeImage($imagickShadow, Imagick::COMPOSITE_OVER, -4, -2);
    $imagickShadow->destroy();

    $imagickDraw->setFillColor($ip_fff);

    if($nowPlaying) {
        $imagickDraw->setFont(FA_FONT_FAM);
        $imagickDraw->setFontSize(NP_FONT_SIZE_ICON);
        $imagick->annotateImage($imagickDraw, 8, SIG_HEIGHT - 9, 0, "\u{f001}");

        $imagickDraw->setFont(NP_FONT_FAM);
        $imagickDraw->setFontSize(NP_FONT_SIZE);
        $imagick->annotateImage($imagickDraw, 32, SIG_HEIGHT - 5, 0, $npInfo->title);

        $imagickDraw->setFontSize(NP_FONT_SIZE_SMALL);
        $imagick->annotateImage($imagickDraw, 32, SIG_HEIGHT - 20, 0, $npInfo->artist);
    } else {
        $imagickDraw->setFont(NP_FONT_FAM);
        $imagickDraw->setFontSize(NP_FONT_SIZE);
        $imagick->annotateImage($imagickDraw, 10, SIG_HEIGHT - 4, 0, "If it ain't broke, I'll break it");
    }

    header('Content-Type: image/png');
    header('Cache-Control: max-age=' . SIG_AGE . ', public');
    header('Content-Disposition: inline; filename="signature.png"');
    echo $imagick->getImageBlob();

    $imagick->destroy();
    return;
}

if($reqPath === '/') {
    $coords = explode(',', (string)filter_input(INPUT_SERVER, 'QUERY_STRING'), 3);
    if(count($coords) === 2) {
        $x = max(0, min(SIG_WIDTH, (int)$coords[0]));
        $y = max(0, min(SIG_HEIGHT, (int)$coords[1]));
    } else $x = $y = 0;

    http_response_code(302);

    if($nowPlaying && (($y >= (SIG_HEIGHT - 33)) || ($nowPlayingHasCover && $x >= (SIG_WIDTH - SIG_HEIGHT)))) {
        header('Location: ' . $npInfo->url);
        return;
    }

    header('Location: https://flash.moe');
    return;
}

http_response_code(404);
echo '404';
